from django.contrib import admin, messages
from django.utils.translation import ngettext
from outbreak_tracker.models import Disease, TrackerLog, Demographics, FAQSection, FAQDetail


admin.site.site_header = 'Caribbean Virus Tracker Administration Portal'
admin.site.site_title = 'Admin Portal'
admin.site.index_title = 'Caribbean Virus Tracker'

@admin.register(Disease)
class DiseaseAdmin(admin.ModelAdmin):
    list_display = ('code_name','name', 'start_date', 'end_date',)
    list_filter = ('start_date', 'end_date')
    fieldsets = (
        ('General', {
            'fields':('code_name','name', 'description','reference_url',)
        }),
        ('Pandemic Period', {
            'fields':('start_date','end_date')
        }),
        ('Read Only', {
            'classes':('collapse',),
            'fields':('created', 'last_updated')
        }),
    )
    readonly_fields = ('created', 'last_updated',)


@admin.register(TrackerLog)
class TrackerLogAdmin(admin.ModelAdmin):
    list_display = ('id', 'date', 'country', 'is_published' , 'total_cases', 'total_deaths', 'recovered')
    list_filter = ('date','is_published')
    ordering = ('-date','country','is_published')
    fieldsets = (
        ('General', {
            'fields':('disease', 'country',)
        }),
        ('Case Stats', {
            'fields':('date', ('total_cases', 'total_deaths', 'recovered',), ('source_name', 'source_url',),)
        }),
        ('Read Only', {
            'classes':('collapse',),
            'fields':(('is_published', 'created', 'last_updated',),)
        }),
    )
    readonly_fields = ('is_published','created', 'last_updated',)
    actions = ['make_published']

    def make_published(self, request, queryset):
        queryset.update(is_published=True)
        self.message_user(request, ngettext(
            '%d country logs was successfully marked as published.',
            '%d countries logs were successfully marked as published.',
            queryset.count()
        ) % queryset.count(), messages.SUCCESS)
    make_published.short_description = 'Publish Tracker Logs'


@admin.register(Demographics)
class DemographicsAdmin(admin.ModelAdmin):
    list_display = ('country', 'latitude', 'longitude', 'parent_country', 'last_updated')
    ordering = ('country',)
    list_filter = ('parent_country',)


class FAQDetailInline(admin.StackedInline):
    model = FAQDetail
    fields = ('question', 'answer', 'priority')
    extra = 1


@admin.register(FAQSection)
class FAQSectionAdmin(admin.ModelAdmin):
    list_display = ('title', 'disease', 'priority')
    list_filter = ('disease',)
    ordering = ('priority',)
    inlines = [FAQDetailInline,]




from django.db import models
from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _
from django_countries import Countries
from django_countries.fields import CountryField
import datetime

class CaribbeanCountries(Countries):
    only = [
        ('AI',_('Anguilla')),
        ('AG',_('Antigua and Barbuda')),
        ('AW',_('Aruba')),
        ('BS',_('Bahamas')),
        ('BB',_('Barbados')),
        ('BZ',_('Belize')),
        ('BM',_('Bermuda')),
        ('BQ',_('Bonaire, Sint Eustatius and Saba')),
        ('VG',_('British Virgin Islands')),
        ('KY',_('Cayman Islands')),
        ('CU',_('Cuba')),
        ('CW',_('Curacao')),
        ('DM',_('Dominica')),
        ('DO',_('Dominican Republic')),
        ('GF',_('French Guiana')),
        ('GD',_('Grenada')),
        ('GP',_('Guadeloupe')),
        ('GY',_('Guyana')),
        ('HT',_('Haiti')),
        ('JM',_('Jamaica')),
        ('MQ',_('Martinique')),
        ('MS',_('Montserrat')),
        ('PR',_('Puerto Rico')),
        ('BL',_('Saint Barthelemy')),
        ('KN',_('Saint Kitts and Nevis')),
        ('LC',_('Saint Lucia')),
        ('MF',_('Saint Martin')),
        ('VC',_('Saint Vincent and the Grenadines')),
        ('SX',_('Sint Maarten')),
        ('SR',_('Suriname')),
        ('TT',_('Trinidad and Tobago')),
        ('TC',_('Turks and Caicos Islands')),
        ('VI',_('United States Virgin Islands')),
    ]


class ParentCountries(Countries):
    only = ['FR', 'NL', 'US', 'GB']


class Demographics(models.Model):
    country = CountryField(countries=CaribbeanCountries, unique=True)
    parent_country = CountryField(countries=ParentCountries, null=True, blank=True)
    year_of_data = models.PositiveIntegerField(null=False, blank=False)
    country_area = models.PositiveIntegerField(null=False, blank=False)
    latitude = models.DecimalField(null=False, blank=False, default=0, max_digits=9, decimal_places=6)
    longitude = models.DecimalField(null=False, blank=False, default=0, max_digits=9, decimal_places=6)
    country_population = models.PositiveIntegerField(null=False, blank=False)
    caricom = models.BooleanField(default=False)
    last_updated = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        verbose_name_plural = _('Demographics')

    def __str__(self):
        return '%s (%s)' % (self.country.name,self.country)


class Disease(models.Model):
    name = models.CharField(max_length=30, null=False, blank=False)
    description = models.TextField(max_length=1000,null=False, blank=False)
    code_name = models.CharField(max_length=30,null=False, blank=False)
    reference_url = models.URLField(null=True, blank=True)
    start_date = models.DateField(null=False, blank=False)
    end_date = models.DateField(null=True, blank=True)
    last_updated = models.DateTimeField(auto_now=True, editable=False)
    created = models.DateTimeField(auto_now_add=True, editable=False)

    class Meta:
        verbose_name_plural = _('Diseases')

    def __str__(self):
        return '%s (%s)' % (self.name, self.code_name)


class TrackerLog(models.Model):
    disease = models.ForeignKey(Disease, related_name='disease_tracker_logs', on_delete=models.CASCADE)
    country = models.ForeignKey(Demographics, related_name='country_tracker_logs', on_delete=models.CASCADE)
    date = models.DateField(null=False)
    total_cases = models.PositiveIntegerField(default=0)
    total_new_cases = models.IntegerField(default=0, editable=False)
    total_deaths = models.PositiveIntegerField(default=0)
    total_new_deaths = models.IntegerField(default=0, editable=False)
    recovered = models.PositiveIntegerField(default=0)
    incidence_rate = models.DecimalField(default=0, max_digits=7, decimal_places=2, editable=False)
    case_fatality_ratio = models.DecimalField(default=0, max_digits=7, decimal_places=2, editable=False)
    source_name = models.CharField(max_length=30, null=False)
    source_url = models.URLField()
    is_published = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True, editable=False)
    last_updated = models.DateTimeField(auto_now=True, editable=False)
    
    class Meta:
        verbose_name_plural = _('Tracker Logs')

    def __str__(self):
        return '%s Log #%s for %s [%s]' % (self.disease.code_name, str(self.pk), self.country.country.name, self.date)


class FAQSection(models.Model):
    disease = models.ForeignKey(Disease, on_delete=models.CASCADE)
    title = models.CharField(max_length=60, null=False, blank=False)
    priority = models.PositiveIntegerField(default=1)

    class Meta:
        verbose_name_plural = _('FAQ Sections')

    def __str__(self):
        return 'Section #%s - %s [%s]' % (str(self.pk), self.title, self.disease.code_name)


class FAQDetail(models.Model):
    section = models.ForeignKey(FAQSection, on_delete=models.CASCADE)
    question = models.CharField(max_length=120, null=False, blank=False)
    answer = models.TextField(null=False, blank=False)
    priority = models.PositiveIntegerField(default=1)

    class Meta:
        verbose_name_plural = _('FAQ Details')

    def __str__(self):
        return '#%s - %s' % (str(self.pk), self.question)


@receiver(pre_save, sender=TrackerLog)
def auto_calculate(sender, instance, *args, **kwargs):
    try:
        previous_date = instance.date - datetime.timedelta(days=1)
        previous_rec = TrackerLog.objects.get(date=previous_date,disease__pk=instance.disease.pk,country__pk=instance.country.pk)
        instance.total_new_cases = instance.total_cases - previous_rec.total_cases
        instance.total_new_deaths = instance.total_deaths - previous_rec.total_deaths
    except TrackerLog.DoesNotExist:
        instance.total_new_cases = 0
        instance.total_new_deaths = 0
    except:
        pass

    population = Demographics.objects.get(pk=instance.country.pk).country_population
    try:
        instance.incidence_rate = (float(instance.total_cases) / float(population)) * 100000
    except ZeroDivisionError:
        instance.incidence_rate = 0
    except:
        pass

    try:
        instance.case_fatality_ratio = (instance.total_deaths / instance.total_cases) * 100
    except ZeroDivisionError:
        instance.case_fatality_ratio = 0
    except:
        pass    



from setuptools import setup, find_packages

version=__import__('outbreak_tracker').__version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="caribbean-outbreak-tracker",
    version=version,
    author="Obika Gellineau",
    author_email="obikagellineau@gmail.com",
    description="Django package for tracking pandemic outbreaks in the Caribbean.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/obikag/caribbean-outbreak-tracker",
    packages=find_packages(exclude=["testproject"]),
    install_requires=[
        "Django>=2.0",
        "django-countries>=6.0,<7.0"
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Healthcare Industry",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Environment :: Web Environment",
        "Topic :: Internet",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content"
    ],
    python_requires='>=3.6',
)


from django import template

register = template.Library()

@register.simple_tag(name='grand_total_cases')
def total_cases(virus_code):
    return virus_code

@register.simple_tag(name='grand_total_new_cases')
def total_new_cases(virus_code):
    return virus_code

@register.simple_tag(name='grand_total_deaths')
def total_deaths(virus_code):
    return virus_code

@register.simple_tag(name='grand_total_new_deaths')
def total_new_deaths(virus_code):
    return virus_code

@register.simple_tag(name='grand_total_recovered')
def recovered(virus_code):
    return virus_code